@@ -570,12 +570,16 @@ class Record:
         value: Any = None,
         type: str = None,
         event: str = None,
+        **kwargs,
     ):
+        if not key:
+            raise ValueError(f"Record cannot be created with empty key '{key}'.")
 
-        self.key = key
+        self.key = safe_str(key)
         self.value = value
-        self.type = type
-        self.event = event
+        self.type = safe_str(type)
+        self.event = safe_str(event)
+        self.kwargs = kwargs
 
 
 class StreamRecord:
@@ -602,17 +606,22 @@ class StreamRecord:
         key: str,
         id: str = None,
         value: Any = None,
+        **kwargs,
     ):
-        self.key = key
-        self.id = id
+        if not key:
+            raise ValueError(f"Record cannot be created with empty key '{key}'.")
+
+        self.key = safe_str(key)
+        self.id = safe_str(id) if id else None
         self.value = value
+        self.kwargs = kwargs
 
 
-def record(rec: Union[str, dict]) -> Record:
+def record(rec: Any) -> Record:
     """Create a Record
 
     Args:
-        rec (Union[str, dict]):
+        rec (Any):
             the value to parse. Either a string (key only) or a dict with at minimum
             the key `key` present and optionally any of the keys `value`, `type`
             and/or `event`.
@@ -627,15 +636,14 @@ def record(rec: Union[str, dict]) -> Record:
     if isinstance(rec, dict):
         return Record(**rec)
 
-    if isinstance(rec, str):
-        return Record(rec)
+    return Record(rec)
 
 
-def stream_record(rec: dict) -> StreamRecord:
+def stream_record(rec: Any) -> StreamRecord:
     """Create a Record
 
     Args:
-        rec (Union[str, dict]):
+        rec (Any):
             the value to parse. Either a string (key only) or a dict with at minimum
             the key `key` present and optionally any of the keys `value`, `type`
             and/or `event`.
@@ -647,7 +655,10 @@ def stream_record(rec: dict) -> StreamRecord:
     if isinstance(rec, StreamRecord):
         return rec
 
-    return StreamRecord(**rec)
+    if isinstance(rec, dict):
+        return StreamRecord(**rec)
+
+    return StreamRecord(rec)
 
 
 def compose(*function: Callable) -> Callable:
@@ -712,3 +723,46 @@ def dict_filter(**kwargs) -> Filterer[Dict[str, Any]]:
         return True
 
     return predicate
+
+
+def passfun(fun: Optional[T] = None, default: Optional[T] = None) -> T:
+    """Create a Python 'function' object from any 'Callable', or constant.
+
+    RedisGears operator callbacks ony accept proper 'function's and not every type of
+    'Callable', such as for example 'method's (e.g. `redgrease.cmd.incr`) or
+    'method-desriptor's (e.g. `str.split`), wich forces users to write seemingly
+    "unecceary" lambda-functions to wrap these.
+
+    This function ensures that the argument is a proper 'function', and thus will be
+    accepted as RedisGears operator callback (asuming the type signature is correct).
+
+    It can also be used to create 'constant'-functions, if passing a non-callable,
+    or to create the 'identity-function`, if called with no arguments.
+
+
+    Args:
+        fun (Optional[T], optional):
+            Callable to turn into a 'function`
+            Alternatively a constant, to use to create a constant function,
+            i.e. a function that alway return the same thing, regarding of input.
+            If None, and no default, the 'identity-function' is returned.
+            Defaults to None.
+
+        default (Optional[T], optional):
+            Default Callable to use as fallback if the 'fun' argument isn't a callable.
+            Defaults to None.
+
+    Returns:
+        T:
+            [description]
+    """
+    if fun is None:
+        return passfun(default) if default else lambda x: x  # type: ignore
+
+    if type(fun).__name__ == "function":
+        return fun
+
+    if callable(fun):
+        return lambda *a, **kw: fun(*a, **kw)  # type: ignore
+
+    return passfun(default) if default else lambda *__, **_: fun  # type: ignore
